// NOTE: Autogenerated file

#if defined(DEFINE_CLOUD)
/**
 * See `Token@index_id`
 */
#define INDEX_ID_LENGTH 5
#endif

#if defined(DEFINE_CLOUD)
/**
 * The callback signature is a kmac of the body of the request.
 * It is used to assert the client can call this callback.
 */
#define CALLBACK_SIGNATURE_LENGTH 32
#endif

#if defined(DEFINE_CLOUD)
/**
 * The number of seconds of validity of the requests to the Findex Cloud
 * backend. After this time, the request cannot be accepted by the backend.
 * This is done to prevent replay attacks.
 */
#define REQUEST_SIGNATURE_TIMEOUT_AS_SECS 60
#endif

#if defined(DEFINE_CLOUD)
/**
 * This seed is used to derive a new 32 bytes Kmac key.
 */
#define SIGNATURE_SEED_LENGTH 16
#endif

/**
 * Limit on the recursion to use when none is provided.
 */
#define MAX_DEPTH 100

/**
 * A pagination is performed in order to fetch the entire Entry Table. It is
 * fetched by batches of size [`NUMBER_OF_ENTRY_TABLE_LINE_IN_BATCH`].
 */
#define NUMBER_OF_ENTRY_TABLE_LINE_IN_BATCH 100

/**
 * See [`FindexCallbacks::progress()`](cosmian_findex::FindexCallbacks::progress).
 *
 * # Serialization
 *
 * The intermediate results are serialized as follows:
 *
 * `LEB128(n_keywords) || LEB128(keyword_1)
 *     || keyword_1 || LEB128(n_associated_results)
 *     || LEB128(associated_result_1) || associated_result_1
 *     || ...`
 *
 * With the serialization of a keyword being:
 *
 * `LEB128(keyword.len()) || keyword`
 *
 * the serialization of the values associated to a keyword:
 *
 * `LEB128(serialized_results_for_keyword.len()) || serialized_result_1 || ...`
 *
 * and the serialization of a result:
 *
 * `LEB128(byte_vector.len() + 1) || prefix || byte_vector`
 *
 * where `prefix` is `l` (only `Location`s are returned) and the `byte_vector`
 * is the byte representation of the location.
 */
typedef int (*ProgressCallback)(const unsigned char *intermediate_results_ptr, unsigned int intermediate_results_len);

/**
 * See [`FindexCallbacks::fetch_entry_table()`](cosmian_findex::FindexCallbacks::fetch_entry_table).
 *
 * # Serialization
 *
 * The input is serialized as follows:
 *
 * `LEB128(n_uids) || UID_1 || ...`
 *
 * The output should be deserialized as follows:
 *
 * `LEB128(n_entries) || UID_1 || LEB128(value_1.len()) || value_1 || ...`
 */
typedef int (*FetchEntryTableCallback)(unsigned char *entries_ptr, unsigned int *entries_len, const unsigned char *uids_ptr, unsigned int uids_len);

/**
 * See [`FindexCallbacks::fetch_chain_table()`](cosmian_findex::FindexCallbacks::fetch_chain_table).
 *
 * # Serialization
 *
 * The input is serialized as follows:
 *
 * `LEB128(n_uids) || UID_1 || ...`
 *
 * The output should be serialized as follows:
 *
 * `LEB128(n_lines) || UID_1 || LEB128(value_1.len()) || value_1 || ...`
 */
typedef int (*FetchChainTableCallback)(unsigned char *chains_ptr, unsigned int *chains_len, const unsigned char *uids_ptr, unsigned int uids_len);

/**
 * See [`FindexCallbacks::upsert_entry_table()`](cosmian_findex::FindexCallbacks::upsert_entry_table).
 *
 * # Serialization
 *
 * The input is serialized as follows:
 *
 * ` LEB128(entries.len()) || UID_1
 *     || LEB128(old_value_1.len()) || old_value_1
 *     || LEB128(new_value_1.len()) || new_value_1
 *     || ...`
 *
 * The output should be serialized as follows:
 *
 * `LEB128(n_lines) || UID_1 || LEB128(value_1.len()) || value_1 || ...`
 */
typedef int (*UpsertEntryTableCallback)(unsigned char *outputs_ptr, unsigned int *outputs_len, const unsigned char *entries_ptr, unsigned int entries_len);

/**
 * See [`FindexCallbacks::insert_chain_table()`](cosmian_findex::FindexCallbacks::insert_chain_table).
 *
 * # Serialization
 *
 * The input is serialized as follows:
 *
 * `LEB128(n_lines) || UID_1 || LEB128(value_1.len() || value_1 || ...`
 */
typedef int (*InsertChainTableCallback)(const unsigned char *chains_ptr, unsigned int chains_len);

/**
 * See [`FindexCallbacks::fetch_all_entry_table_uids()`](cosmian_findex::FindexCallbacks::fetch_all_entry_table_uids).
 *
 * The output should be deserialized as follows:
 *
 * `UID_1 || UID_2 || ... || UID_n`
 */
typedef int (*FetchAllEntryTableUidsCallback)(unsigned char *uids_ptr, unsigned int *uids_len);

/**
 * See [`FindexCallbacks::update_lines()`](cosmian_findex::FindexCallbacks::update_lines).
 *
 * # Serialization
 *
 * The removed Chain Table UIDs are serialized as follows:
 *
 * `LEB128(n_uids) || UID_1 || ...`
 *
 * The new table items are serialized as follows:
 *
 * `LEB128(n_items) || UID_1 || LEB128(value_1.len()) || value_1 || ...`
 */
typedef int (*UpdateLinesCallback)(const unsigned char *chain_table_uids_to_remove_ptr, unsigned int chain_table_uids_to_remove_len, const unsigned char *new_encrypted_entry_table_items_ptr, unsigned int new_encrypted_entry_table_items_len, const unsigned char *new_encrypted_chain_table_items_ptr, unsigned int new_encrypted_chain_table_items_len);

/**
 * See
 * [`FindexCallbacks::list_removed_locations()`](cosmian_findex::FindexCallbacks::list_removed_locations).
 *
 * # Serialization
 *
 * The input is serialized as follows:
 *
 * `LEB128(locations.len()) || LEB128(location_bytes_1.len()
 *     || location_bytes_1 || ...`
 *
 * Outputs should follow the same serialization.
 */
typedef int (*ListRemovedLocationsCallback)(unsigned char *removed_locations_ptr, unsigned int *removed_locations_len, const unsigned char *locations_ptr, unsigned int locations_len);

/**
 * Re-export the `cosmian_ffi` `h_get_error` function to clients with the old
 * `get_last_error` name The `h_get_error` is available inside the final lib
 * (but tools like ffigen seems to not parse it…) Maybe we can find a solution
 * by changing the function name inside the clients.
 *
 * # Safety
 *
 * It's unsafe.
 */
int get_last_error(char *error_ptr, int *error_len);

/**
 * Recursively searches Findex graphs for values indexed by the given keywords.
 *
 * # Serialization
 *
 * Le output is serialized as follows:
 *
 * `LEB128(n_keywords) || LEB128(keyword_1)
 *     || keyword_1 || LEB128(n_associated_results)
 *     || LEB128(associated_result_1) || associated_result_1
 *     || ...`
 *
 * # Parameters
 *
 * - `search_results`            : (output) search result
 * - `master_key`                : master key
 * - `label`                     : additional information used to derive Entry
 *   Table UIDs
 * - `keywords`                  : `serde` serialized list of base64 keywords
 * - `max_results_per_keyword`   : maximum number of results returned per
 *   keyword
 * - `max_depth`                 : maximum recursion depth allowed
 * - `fetch_chains_batch_size`   : increase this value to improve performances
 *   but decrease security by batching fetch chains calls
 * - `progress_callback`         : callback used to retrieve intermediate
 *   results and transmit user interrupt
 * - `fetch_entry_callback`      : callback used to fetch the Entry Table
 * - `fetch_chain_callback`      : callback used to fetch the Chain Table
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_search(char *search_results_ptr,
             int *search_results_len,
             const char *master_key_ptr,
             int master_key_len,
             const uint8_t *label_ptr,
             int label_len,
             const char *keywords_ptr,
             int max_results_per_keyword,
             int max_depth,
             unsigned int fetch_chains_batch_size,
             ProgressCallback progress_callback,
             FetchEntryTableCallback fetch_entry_callback,
             FetchChainTableCallback fetch_chain_callback);

/**
 * Index the given values for the given keywords. After upserting, any
 * search for such a keyword will result in finding (at least) the
 * corresponding value.
 *
 * # Serialization
 *
 * The list of values to index for the associated keywords should be serialized
 * as follows:
 *
 * `LEB128(n_values) || serialized_value_1
 *     || LEB128(n_associated_keywords) || serialized_keyword_1 || ...`
 *
 * where values serialized as follows:
 *
 * `LEB128(value_bytes.len() + 1) || base64(prefix || value_bytes)`
 *
 * with `prefix` being `l` for a `Location` and `w` for a `NextKeyword`, and
 * where keywords are serialized as follows:
 *
 * `LEB128(keyword_bytes.len()) || base64(keyword_bytes)`
 *
 * # Parameters
 *
 * - `master_key`      : Findex master key
 * - `label`           : additional information used to derive Entry Table UIDs
 * - `indexed_values_and_keywords` : serialized list of values and the keywords
 *   used to index them
 * - `fetch_entry`     : callback used to fetch the Entry Table
 * - `upsert_entry`    : callback used to upsert lines in the Entry Table
 * - `insert_chain`    : callback used to insert lines in the Chain Table
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_upsert(const uint8_t *master_key_ptr,
             int master_key_len,
             const uint8_t *label_ptr,
             int label_len,
             const char *indexed_values_and_keywords_ptr,
             FetchEntryTableCallback fetch_entry,
             UpsertEntryTableCallback upsert_entry,
             InsertChainTableCallback insert_chain);

/**
 * Replaces all the Index Entry Table UIDs and values. New UIDs are derived
 * using the given label and the KMAC key derived from the new master key. The
 * values are decrypted using the DEM key derived from the master key and
 * re-encrypted using the DEM key derived from the new master key.
 *
 * Randomly selects index entries and recompact their associated chains. Chains
 * indexing no existing location are removed. Others are recomputed from a new
 * keying material. This removes unneeded paddings. New UIDs are derived for
 * the chain and values are re-encrypted using a DEM key derived from the new
 * keying material.
 *
 * # Parameters
 *
 * - `num_reindexing_before_full_set`  : number of compact operation needed to
 *   compact all Chain Table
 * - `old_master_key`                  : old Findex master key
 * - `new_master_key`                  : new Findex master key
 * - `new_label`                       : additional information used to derive
 *   Entry Table UIDs
 * - `fetch_entry`                     : callback used to fetch the Entry Table
 * - `fetch_chain`                     : callback used to fetch the Chain Table
 * - `update_lines`                    : callback used to update lines in both
 *   tables
 * - `list_removed_locations`          : callback used to list removed
 *   locations among the ones given
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_compact(int num_reindexing_before_full_set,
              const uint8_t *old_master_key_ptr,
              int old_master_key_len,
              const uint8_t *new_master_key_ptr,
              int new_master_key_len,
              const uint8_t *new_label_ptr,
              int new_label_len,
              FetchAllEntryTableUidsCallback fetch_all_entry_table_uids,
              FetchEntryTableCallback fetch_entry,
              FetchChainTableCallback fetch_chain,
              UpdateLinesCallback update_lines,
              ListRemovedLocationsCallback list_removed_locations);

#if defined(DEFINE_CLOUD)
/**
 * Recursively searches Findex graphs for values indexed by the given keywords.
 *
 * # Serialization
 *
 * Le output is serialized as follows:
 *
 * `LEB128(n_keywords) || LEB128(keyword_1)
 *     || keyword_1 || LEB128(n_associated_results)
 *     || LEB128(associated_result_1) || associated_result_1
 *     || ...`
 *
 * # Parameters
 *
 * - `search_results`            : (output) search result
 * - `token`                     : findex cloud token
 * - `label`                     : additional information used to derive Entry
 *   Table UIDs
 * - `keywords`                  : `serde` serialized list of base64 keywords
 * - `max_results_per_keyword`   : maximum number of results returned per
 *   keyword
 * - `max_depth`                 : maximum recursion depth allowed
 * - `fetch_chains_batch_size`   : increase this value to improve performances
 *   but decrease security by batching fetch chains calls
 * - `base_url`                  : base URL for Findex Cloud (with http prefix
 *   and port if required). If null, use the default Findex Cloud server.
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_search_cloud(char *search_results_ptr,
                   int *search_results_len,
                   const char *token_ptr,
                   const uint8_t *label_ptr,
                   int label_len,
                   const char *keywords_ptr,
                   int max_results_per_keyword,
                   int max_depth,
                   unsigned int fetch_chains_batch_size,
                   const char *base_url_ptr);
#endif

#if defined(DEFINE_CLOUD)
/**
 * Index the given values for the given keywords. After upserting, any
 * search for such a keyword will result in finding (at least) the
 * corresponding value.
 *
 * # Serialization
 *
 * The list of values to index for the associated keywords should be serialized
 * as follows:
 *
 * `LEB128(n_values) || serialized_value_1
 *     || LEB128(n_associated_keywords) || serialized_keyword_1 || ...`
 *
 * where values serialized as follows:
 *
 * `LEB128(value_bytes.len() + 1) || base64(prefix || value_bytes)`
 *
 * with `prefix` being `l` for a `Location` and `w` for a `NextKeyword`, and
 * where keywords are serialized as follows:
 *
 * `LEB128(keyword_bytes.len()) || base64(keyword_bytes)`
 *
 * # Parameters
 *
 * - `token`           : Findex Cloud token
 * - `label`           : additional information used to derive Entry Table UIDs
 * - `indexed_values_and_keywords` : serialized list of values and the keywords
 *   used to index them
 * - `base_url`                  : base URL for Findex Cloud (with http prefix
 *   and port if required). If null, use the default Findex Cloud server.
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_upsert_cloud(const char *token_ptr,
                   const uint8_t *label_ptr,
                   int label_len,
                   const char *indexed_values_and_keywords_ptr,
                   const char *base_url_ptr);
#endif

#if defined(DEFINE_CLOUD)
/**
 * Generate a new Findex token from the provided index ID and signature seeds,
 * and a randomly generated Findex master key inside Rust.
 *
 * The token is output inside `token_ptr`, `token_len` is updated to match the
 * token length (this length should always be the same, right now, the length
 * is always below 200 bytes)
 *
 * # Safety
 *
 * Cannot be safe since using FFI.
 */
int h_generate_new_token(uint8_t *token_ptr,
                         int *token_len,
                         const char *index_id_ptr,
                         const uint8_t *fetch_entries_seed_ptr,
                         int fetch_entries_seed_len,
                         const uint8_t *fetch_chains_seed_ptr,
                         int fetch_chains_seed_len,
                         const uint8_t *upsert_entries_seed_ptr,
                         int upsert_entries_seed_len,
                         const uint8_t *insert_chains_seed_ptr,
                         int insert_chains_seed_len);
#endif
